import 'babel-polyfill';

import React, { Component } from 'react';
import PropTypes from 'prop-types';

import search from './search';

import {
  Input,
  InputWrapper,
  SearchIcon,
  ResultList,
  Result,
  ResultLink,
  Wrapper,
} from './styles';

export default class Geocoder extends Component {
  constructor() {
    super();

    this.state = {
      results: [],
      focus: null,
      loading: false,
      searchTime: new Date(),
      value: '',
    };

    this.input = React.createRef();
    this.onInput = this.onInput.bind(this);
    this.onKeyDown = this.onKeyDown.bind(this);
    this.clickOption = this.clickOption.bind(this);
    this.onResult = this.onResult.bind(this);
    this.onChange = this.onChange.bind(this);
    this.onSelect = this.onSelect.bind(this);
  }

  componentDidMount() {
    const { focusOnMount } = this.props;
    if (focusOnMount) this.input.current.focus();
  }

  onChange({ target }) {
    this.setState({
      value: target.value,
    });
  }

  onInput(e) {
    this.setState({ loading: true });
    const { value } = e.target;
    if (value === '') {
      this.setState({
        results: [],
        focus: null,
        loading: false,
      });
    } else {
      const {
        endpoint,
        source,
        accessToken,
        proximity,
        bbox,
        types,
      } = this.props;

      search({
        endpoint,
        source,
        accessToken,
        proximity,
        bbox,
        types,
        query: value,
        onResult: this.onResult,
      });
    }
  }

  onKeyDown(e) {
    const { results, focus } = this.state;
    switch (e.which) {
      // up
      case 38:
        e.preventDefault();
        this.moveFocus(-1);
        break;
      // down
      case 40:
        this.moveFocus(1);
        break;
      // accept
      case 13:
        if (results.length > 0 && focus == null) {
          this.clickOption(results[0], 0);
        }
        this.acceptFocus();
        break;
      default:
        break;
    }
  }

  onResult(body, searchTime) {
    const { searchTime: oldSearchTime, results } = this.state;
    const { onSuggest } = this.props;

    // searchTime is compared with the last search to set the state
    // to ensure that a slow xhr response does not scramble the
    // sequence of autocomplete display.
    if (body && body.features && oldSearchTime <= searchTime) {
      this.setState({
        searchTime,
        loading: false,
        results: body.features,
        focus: null,
      });
      onSuggest(results);
    }
  }

  onSelect(place) {
    const { onSelect } = this.props;

    this.setState({
      value: '',
      results: [],
    });

    onSelect(place);
  }

  setInput(text) {
    this.setState({
      value: text,
    });
  }

  moveFocus(dir) {
    const { loading, focus, results } = this.state;
    if (loading) return;
    this.setState({
      focus: focus === null
        ? 0 : Math.max(0,
          Math.min(
            results.length - 1,
            focus + dir,
          )),
    });
  }

  acceptFocus() {
    const { focus, results } = this.state;
    const { onSelect } = this.props;

    if (focus !== null) {
      this.onSelect(results[focus]);
    }
  }

  clickOption(place, listLocation) {
    const { onSelect } = this.props;

    this.onSelect(place);
    this.setState({ focus: listLocation });

    // focus on the input after click to maintain key traversal
    this.input.current.focus();
    return false;
  }

  render() {
    const {
      inputClass,
      inputPlaceholder,
      inputPosition,
      showLoader,
      className,
      resultsClass,
      resultClass,
      resultFocusClass,
      ResultList: CustomResultList,
      Result: CustomResult,
      ResultLink: CustomResultLink,
      InputWrapper: CustomInputWrapper,
      SearchIcon: CustomSearchIcon,
      Input: CustomInput,
      Wrapper: CustomWrapper,
    } = this.props;

    const {
      results,
      loading,
      focus,
      value,
    } = this.state;

    const Components = {
      ResultList: CustomResultList || ResultList,
      Result: CustomResult || Result,
      ResultLink: CustomResultLink || ResultLink,
      InputWrapper: CustomInputWrapper || InputWrapper,
      SearchIcon: CustomSearchIcon || SearchIcon,
      Input: CustomInput || Input,
      Wrapper: CustomWrapper || Wrapper,
    };

    const input = (
      <Components.InputWrapper>
        <Components.SearchIcon />
        <Components.Input
          ref={this.input}
          className={inputClass}
          onInput={this.onInput}
          onKeyDown={this.onKeyDown}
          onChange={this.onChange}
          placeholder={inputPlaceholder}
          type="text"
          value={value}
        />
      </Components.InputWrapper>
    );

    return (
      <Components.Wrapper className={className}>
        {inputPosition === 'top' && input}
        <Components.ResultList className={`
              ${showLoader && loading
          ? 'loading'
          : ''
              }
              ${resultsClass}`}
        >
          {results.length > 0 && (
            results.map((result, i) => (
              <Components.Result key={result.id}>
                <Components.ResultLink
                  type="button"
                  onClick={() => this.clickOption(result, i)}
                  className={`${resultClass} ${i === focus ? resultFocusClass : ''}`}
                  key={result.id}
                >
                  {result.place_name}
                </Components.ResultLink>
              </Components.Result>
            ))
          )}
        </Components.ResultList>
        {inputPosition === 'bottom' && input}
      </Components.Wrapper>
    );
  }
}

Geocoder.propTypes = {
  endpoint: PropTypes.string,
  source: PropTypes.string,
  accessToken: PropTypes.string.isRequired,

  className: PropTypes.string,
  inputClass: PropTypes.string,
  resultClass: PropTypes.string,
  resultsClass: PropTypes.string,
  resultFocusClass: PropTypes.string,

  ResultList: PropTypes.element,
  Result: PropTypes.element,
  ResultLink: PropTypes.element,
  InputWrapper: PropTypes.element,
  SearchIcon: PropTypes.element,
  Input: PropTypes.element,
  Wrapper: PropTypes.element,

  showLoader: PropTypes.bool,
  focusOnMount: PropTypes.bool,

  inputPosition: PropTypes.string,
  inputPlaceholder: PropTypes.string,

  onSelect: PropTypes.func.isRequired,
  onSuggest: PropTypes.func,

  proximity: PropTypes.string,
  bbox: PropTypes.arrayOf(PropTypes.number),

  types: PropTypes.string,
};

Geocoder.defaultProps = {
  endpoint: 'https://api.tiles.mapbox.com',

  inputClass: '',
  resultClass: '',
  resultsClass: '',
  resultFocusClass: 'strong',

  ResultList: null,
  Result: null,
  ResultLink: null,
  InputWrapper: null,
  SearchIcon: null,
  Input: null,
  Wrapper: null,

  inputPosition: 'top',

  inputPlaceholder: 'Search',

  showLoader: false,

  source: 'mapbox.places',
  proximity: '',

  bbox: [],
  types: '',

  onSuggest: () => null,
  focusOnMount: true,
};
